function [lxo,lyo,lzo]=adj(tstep,h,x,y,z,datx,daty,datz,a,r,b,D)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Purpose: Calculate gradient of cost function by means of
%           adjoint of modified Euler scheme
%           
%
%  (c) 2002  Data Assimilation Research Centre
%
%  Original Fortran program by Marek Wlasak
%  Converted to Matlab by Amos Lawless
%
%  List of main variables
%    a:          sigma coefficient in equations
%    r:          rho coefficient in equations
%    b:          beta coefficient in equations
%    D:          Observation weighting matrix
%    h:          Time step for numerical scheme
%    tstep:      Number of time steps to perform
%    [x,y,z]:    Forward trajectory
%    [datx,daty,datz]: Observation values
%
%  Output:
%    [lxo,lyo,lzo]: Trajectories of gradient fields
%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lx=zeros(tstep);
ly=zeros(tstep);
lz=zeros(tstep);

      lx(tstep+1)=0.0d0;
      ly(tstep+1)=0.0d0;
      lz(tstep+1)=0.0d0;

      for i=tstep:-1:1
% Evolve adjoint equations backwards
         xxdum=1.0d0+0.5d0*h*(-2.0d0*a+h*(a+r-z(i)));
         xydum=0.5d0*h*(r+(1-a*h)*r-h*y(i)*(x(i)+a*h*(-x(i)+y(i))) ...
            -h*(r-z(i))-z(i)-(1.0d0-a*h)*(z(i)+h*(x(i)*y(i)- ...
            b*z(i))));
         xzdum=0.5d0*h*(y(i)-b*h*y(i)+h*(x(i)+ ...
            a*h*(y(i)-x(i)))*(r-z(i))+(1.0d0-a*h)*(y(i)+ ...
            h*(r*x(i)-y(i)-x(i)*z(i))));
         yxdum=0.5d0*h*a*(2.0d0-h-a*h);
         yydum=1.0d0+0.5d0*h*(-2.0d0+h+a*h*r-h*x(i)*(x(i)+a*h* ...
             (y(i)-x(i)))-a*h*(z(i)+ ...
             h*(x(i)*y(i)-b*z(i)))) ;
         yzdum=0.5d0*h*(x(i)-b*h*x(i)+(1.0d0-h)* ...
            (x(i)+a*h*(y(i)-x(i)))+ ...
            a*h*(y(i)+h*(r*x(i)-y(i)-x(i)*z(i))));
         zxdum=-0.5d0*a*h*h*x(i);
         zydum=0.5d0*h*(-x(i)+h*x(i)- ...
            (1.0d0-b*h)*(x(i)+a*h*(y(i)-x(i))));
         zzdum=1.0d0+0.5d0*h*(-2.0d0*b+b*b*h-h*x(i)* ...
            (x(i)+a*h*(y(i)-x(i))));

% Update gradient of cost function
         lx(i)=2.0d0*h*D(i)*(x(i)-datx(i))+lx(i+1)*xxdum+ly(i+1)* ...
              xydum+lz(i+1)*xzdum;
               
         ly(i)=2.0d0*h*D(i)*(y(i)-daty(i))+lx(i+1)*yxdum+ly(i+1)* ...
              yydum+lz(i+1)*yzdum;

         lz(i)=2.0d0*h*D(i)*(z(i)-datz(i))+lx(i+1)*zxdum+ly(i+1)* ...
              zydum+lz(i+1)*zzdum;

      end
lxo=lx;
lyo=ly;
lzo=lz;
