function [xo,yo,zo]=modeuler(tstep,h,j,zval,yval,xval,a,r,b)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Purpose: Apply modified Euler scheme to the  Lorenz equations
%
%  (c) 2002  Data Assimilation Research Centre
%
%  Original Fortran program by Marek Wlasak
%  Converted to Matlab by Amos Lawless
%
%  List of main variables
%    a:          sigma coefficient in equations
%    r:          rho coefficient in equations
%    b:          beta coefficient in equations
%    h:          Time step for numerical scheme
%    tstep:      Number of time steps to perform
%    [xval,yval,zval]: Initial fields
%    j:          Index to pick up correct initial field
%
%  Output:
%    [xo,yo,zo]: Trajectories of evolved fields
%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x=zeros(tstep,1);
y=zeros(tstep,1);
z=zeros(tstep,1);
%
x(1)=xval(j+1);
y(1)=yval(j+1);
z(1)=zval(j+1);
%
for i = 1:tstep
  kx1=h*f(a,x(i),y(i),z(i));
  ky1=h*g(r,x(i),y(i),z(i));
  kz1=h*fg(b,x(i),y(i),z(i));
  kx2=h*f(a,x(i)+kx1,y(i)+ky1,z(i)+kz1);
  ky2=h*g(r,x(i)+kx1,y(i)+ky1,z(i)+kz1);
  kz2=h*fg(b,x(i)+kx1,y(i)+ky1,z(i)+kz1);
  x(i+1)=x(i)+0.5d0*(kx1+kx2);
  y(i+1)=y(i)+0.5d0*(ky1+ky2);
  z(i+1)=z(i)+0.5d0*(kz1+kz2);
end

xo=x;
yo=y;
zo=z;
%
function [f_out]=f(a,xvl,yvl,zvl);
      f_out=a*(yvl-xvl);

function [g_out]=g(r,xvl,yvl,zvl);
      g_out=r*xvl-yvl-xvl*zvl;

function [fg_out]=fg(b,xvl,yvl,zvl);
      fg_out=xvl*yvl-b*zvl;

